function varargout = plot_improvement(relperformance,improvement,varargin)
%% plotHHI(relperformance,improvement,hfig)

% parse inputs
p = inputParser;
p.addOptional('hfig',[]);
p.addOptional('Color',[0 0.447 0.741]);
p.addOptional('Alpha',0.5);
p.addOptional('XLim',[-0.75 0.75]);
p.addOptional('YLim',[-0.25 0.5]);
p.addOptional('PlotType','');
p.addOptional('XBin',-0.45:0.1:0.45);
p.addOptional('BinRemoveOutliers',1);
p.addOptional('LegendLabels','');
p.addOptional('FontSize',12);
p.addOptional('Model',[]);
p.addOptional('LineStyle','-');
p.parse(varargin{:});

if isempty(p.Results.hfig)
    figure('color','w');
else
    figure(p.Results.hfig);
end
set(gcf,'defaultTextInterpreter','latex');

color = p.Results.Color;
alpha = p.Results.Alpha;
if numel(color) > 3
    alpha = color(4);
end
fontsize = p.Results.FontSize;

switch lower(p.Results.PlotType)
    case 'bin'
        % bin data, calculate mean and std
        xbin = p.Results.XBin;
        
        % plot bins
        [mbins,sbinsy,sbinsx,xbins,ybins] = bin_improvement(relperformance(:),improvement(:),xbin,p.Results.BinRemoveOutliers);
        errorbar(xbin,mbins,sbinsy,'.','LineStyle',p.Results.LineStyle,'linewidth',1,'markersize',14,'color',color,'tag','BIN_RELIMP'); hold on;
        errorbar(xbin,mbins,sbinsx,'horizontal','.','LineStyle',p.Results.LineStyle,'linewidth',1,'markersize',14,'color',color);
        
        varargout{1} = xbin;
        varargout{2} = mbins;
        varargout{3} = sbinsy;
        varargout{4} = sbinsx;
        varargout{5} = xbins;
        varargout{6} = ybins;
        
    case 'cov'
        x1 = relperformance(:,1:2:end); x1 = x1(:);
        y1 = improvement(:,1:2:end); y1 = y1(:);
        
        x2 = relperformance(:,2:2:end); x2 = x2(:);
        y2 = improvement(:,2:2:end); y2 = y2(:);
      
        
        c1 = cov(x1,y1,'omitrows'); 
        hp = error_ellipse(c1,'mu',[nanmean(x1) nanmean(y1)]);
        set(hp,'color',color);
        c2 = cov(x2,y2,'omitrows'); 
        hp = error_ellipse(c2,'mu',[nanmean(x2) nanmean(y2)]);
        set(hp,'color',color);
        
        plot(nanmean(x1),nanmean(y1),'^','MarkerFaceColor',color,'Color',color);
        plot(nanmean(x2),nanmean(y2),'v','MarkerFaceColor',color,'Color',color);
        
        errorbar(nanmean(x1),nanmean(y1),nanstd(y1)/sqrt(numel(y1(~isnan(y1)))),'color',color);
        errorbar(nanmean(x1),nanmean(y1),nanstd(x1)/sqrt(numel(x1(~isnan(x1)))),'horizontal','color',color);
        
        errorbar(nanmean(x2),nanmean(y2),nanstd(y2)/sqrt(numel(y2(~isnan(y2)))),'color',color);
        errorbar(nanmean(x2),nanmean(y2),nanstd(x2)/sqrt(numel(x2(~isnan(x2)))),'horizontal','color',color);
    otherwise
        % nothing
end

% plot data points
hs(1) = scatter(reshape(relperformance(:,1:2:end),[],1),reshape(improvement(:,1:2:end),[],1),30,'.','MarkerFaceColor',color(1:3),'MarkerEdgeColor',color(1:3),'MarkeredgeAlpha',alpha,'MarkerfaceAlpha',alpha,'Tag','RELIMP'); hold on; % 'MarkerFaceColor',color
hs(2) = scatter(reshape(relperformance(:,2:2:end),[],1),reshape(improvement(:,2:2:end),[],1),30,'.','MarkerFaceColor',color(1:3),'MarkerEdgeColor',color(1:3),'MarkeredgeAlpha',alpha,'MarkerfaceAlpha',alpha); % 'MarkerFaceColor',color,

uistack(hs,'bottom');


% figure layout
vxlim = p.Results.XLim;
vylim = p.Results.YLim;
set(gca,'xlim',vxlim,'ylim',vylim);

hlines(1) = plot([0 0],ylim,'k','linewidth',0.5);
hlines(2) = plot(xlim,[0 0],'k','linewidth',0.5);
uistack(hlines,'bottom');

xlabel('Relative performance $\left(1-\frac{e_p}{e}\right)$','interpreter','latex'); 
ylabel('Performance improvement $\left(1-\frac{e_i}{e}\right)$','interpreter','latex');

% better/worse partner labels
text(0.05,-0.2,'Better partner','VerticalAlignment','Middle','FontSize',fontsize);
text(-0.05,-0.2,'Worse partner','VerticalAlignment','Middle','HorizontalAlignment','Right','FontSize',fontsize);
arrow([0.275 -0.2],[0.35 -0.2],'Length',8,'linewidth',0.5);
arrow([-0.275 -0.2],[-0.35 -0.2],'Length',8,'linewidth',0.5)
box on

% model fit
if ~isempty(p.Results.Model)
    % prep data (sort)
    x = relperformance(:);
    y = improvement(:);
    [x,I] = sort(x);
    y = y(I);
    
    id = ~isnan(x);
    x = x(id);
    y = y(id);
    
    id = ~isnan(y);
    x = x(id);
    y = y(id);

    xpred = (p.Results.XLim(1):0.001:p.Results.XLim(end)).';
    
    % fit & predict
    fo = fitoptions('Method','NonLinearLeastSquares',...
                    'Robust','bisquare',...
                    'StartPoint',[0 0 0]);
    ft = fittype(p.Results.Model,'options',fo);  

    [fm,gof,output] = fit(x,y,ft);
    [ypredci,ypred] = predint(fm,xpred,0.95,'functional','on');
    
    plot(xpred,ypred,'-','color',color,'linewidth',2);
    xp = [xpred; flipud(xpred)];
    yp = [ypredci(:,1); flipud(ypredci(:,2));];
    patch(xp,yp,1,'FaceColor',color,'FaceAlpha',0.1,'EdgeColor','none'); %,'FaceColor',[0 0.447 0.741],'EdgeColor','k'
    
    model.fm = fm;
    model.gof = gof;
    model.output = output;
    model.ypredci = ypredci;
    model.ypred = ypred;
    model.xpred = xpred;
    
    varargout{6} = model;
end

% legend
if ~isempty(p.Results.LegendLabels)
    h = flipud(findobj(get(gca,'children'),'Tag','BIN_RELIMP'));
    legend(h,p.Results.LegendLabels,'Location','NorthWest','FontSize',fontsize,'interpreter','latex');
end

end

function [mbins,sbinsy,sbinsx,xbins,ybins] = bin_improvement(xdata,ydata,xbin,removeoutliers)
%% [mbins, sbins] = bin_improvement(xdata,ydata,xbin,removeoutliers)

if (nargin < 4)
    removeoutliers = 0;
end

mbins = NaN(size(xbin));
sbinsx = NaN(size(xbin));
sbinsy = NaN(size(xbin));
binstep = mode(diff(xbin));
ybins = {};

% loop through bins, calculate mean and standard error of the mean
for jj = 1:length(xbin)

    % define minedge and maxedge (boundaries of bin)
    minedge = xbin(jj)-0.5*binstep;
    maxedge = xbin(jj)+0.5*binstep;

    if (jj == 1)
        minedge = min(xdata(:));
    end
    if (jj == length(xbin))
        maxedge = max(xdata(:));
    end

    idx = (xdata > minedge) & (xdata < maxedge);
    x = xdata(idx);
    y = ydata(idx);
    
    if removeoutliers && ~any(y)
        try
            y = deleteoutliers(y,0.05,1);
            x = deleteoutliers(x,0.05,1);
        catch
        end
    end
   
    % calculate mean
    mbins(jj) = nanmean(y);
    ybins{jj} = y;
    xbins{jj} = x;
    sbinsy(jj) = nanstd(y) / sqrt(numel(y(~isnan(y))));
    sbinsx(jj) = nanstd(x) / sqrt(numel(x(~isnan(x))));
end

   

end
